<?php

declare(strict_types=1);

namespace Drupal\cron_last_run_manipulate\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\cron_last_run_manipulate\Utility\UtilityHelperService;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Datetime\DateFormatterInterface;

/**
 * Provides a Cron Last Run Manipulate form.
 */
class CronLastRunManipulateForm extends FormBase {
  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The utility helper service.
   *
   * @var \Drupal\Utility\UtilityHelperService
   */
  protected $utilityHelper;

  /**
   * Stores the state storage service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;


  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * Constructs a CronLastRunManipulateForm object.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Drupal\cron_last_run_manipulate\Utility\UtilityHelperService $utility_helper
   *   The utility helper service.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state key value store.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   */
  public function __construct(ModuleHandlerInterface $module_handler, UtilityHelperService $utility_helper, StateInterface $state, DateFormatterInterface $date_formatter) {
    $this->moduleHandler = $module_handler;
    $this->utilityHelper = $utility_helper;
    $this->state = $state;
    $this->dateFormatter = $date_formatter;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'cron_last_run_manipulate_cron_last_run_manipulate';
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('cron_last_run_manipulate.helper'),
      $container->get('state'),
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    if (!$this->moduleHandler->moduleExists('automated_cron')) {
      $form['automated_cron'] = [
        '#markup' => $this->t('Install the <em>Automated Cron</em> module to allow this module to work'),
      ];
    }
    $form['description'] = [
      '#markup' => '<p>' . $this->t('Options are available to update the last cron run time. The selected time interval will be subtracted from the current server time, and the last cron time will be updated accordingly.') . '</p>',
    ];
    $form['field_group'] = [
      '#type' => 'fieldset',
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
    ];
    $form['field_group']['time'] = [
      '#type' => 'select',
      '#title' => $this->t('Update last cron run time'),
      '#description' => $this->t('Please note the time should be greater than the last cron run time and below maximum cron interval time.'),
      '#default_value' => $this->utilityHelper->getSelectedOption(),
      '#options' => $this->utilityHelper->getDefaultTimeOptions(),
      '#required' => TRUE,
    ];

    $form['field_group']['custom_time'] = [
      '#type' => 'number',
      '#title' => $this->t('Custom time'),
      '#description' => $this->t('Please enter the custom time in seconds.'),
      '#default_value' => ($this->utilityHelper->getSelectedOption() == -1) ? $this->state->get('cron_last_run_manipulate.custom_time') : '',
      '#min' => 1,
      '#max' => $this->utilityHelper->getCronMaxIntervalTime(),
      '#states' => [
        'visible' => [
          ':input[name="time"]' => ['value' => -1],
        ],
        'required' => [
          ':input[name="time"]' => ['value' => -1],
        ],
      ],
    ];

    $form['field_group']['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Update'),
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    parent::validateForm($form, $form_state);
    if (!$this->moduleHandler->moduleExists('automated_cron')) {
      $form_state->setErrorByName('time', $this->t('Please enable Automated Cron module to allow this module to work'));
    }
    if ($this->utilityHelper->getCronMaxIntervalTime() < $form_state->getValue('custom_time')) {
      $form_state->setErrorByName('custom_time', $this->t('Time should be less than the maximum cron interval time.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->utilityHelper->setSelectedOption($form_state->getValue('time'));
    if (!empty($form_state->getValue('custom_time'))) {
      $form_state->setValue('time', $form_state->getValue('custom_time'));
      $this->state->set('cron_last_run_manipulate.custom_time', $form_state->getValue('custom_time'));
    }
    $this->utilityHelper->manipulateLastCronRunTime($form_state->getValue('time'));
    $this->messenger()->addStatus('<p>' . $this->t('Cron last run updated to: %time ago.', ['%time' => $this->dateFormatter->formatTimeDiffSince($this->state->get('system.cron_last'))]) . '</p>');
  }

}
