<?php

namespace Drupal\cron_last_run_manipulate\Utility;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Component\Datetime\Time;

/**
 * Class UtilityHelperService.
 *
 * Represents a user in the system.
 */
class UtilityHelperService {
  use StringTranslationTrait;

  /**
   * Stores the state storage service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;


  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\Time
   */
  protected $time;

  /**
   * Constructs a UtilityHelperService object.
   *
   * @param \Drupal\Core\State\StateInterface $state
   *   The state key value store.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Component\Datetime\Time $time
   *   The time service.
   */
  public function __construct(StateInterface $state, DateFormatterInterface $date_formatter, ConfigFactoryInterface $config_factory, Time $time) {
    $this->state = $state;
    $this->dateFormatter = $date_formatter;
    $this->configFactory = $config_factory;
    $this->time = $time;
  }

  /**
   * Retrieves the default time options.
   *
   * @return array
   *   An associative array of default time options.
   */
  public function getDefaultTimeOptions() {
    $options = [60, 300, 600, 900, 1200, 1500, 1800, 2100, 2400, 2700, 3000, 3300, 3600];
    return array_map(function ($interval) {
      return $this->dateFormatter->formatInterval($interval) . ' ' . $this->t('ago');
    }, array_combine($options, $options)) + [
      -1 => $this->t('Choose Custom time'),
    ];
  }

  /**
   * Retrieves the maximum interval time for the cron job.
   *
   * This function returns the maximum interval time that the cron job is
   * allowed to run. It is used to ensure that the cron job does not
   * exceed a certain time limit.
   *
   * @return int
   *   The maximum interval time in seconds.
   */
  public function getCronMaxIntervalTime() {
    return $this->configFactory->get('automated_cron.settings')->get('interval');
  }

  /**
   * Retrieves the last cron run time.
   *
   * @return int
   *   The timestamp of the last cron run.
   */
  public function getLastCronRunTime() {
    return $this->state->get('system.cron_last');
  }

  /**
   * Manipulates the last cron run time.
   *
   * @param int $time
   *   The timestamp of the last cron run.
   */
  public function manipulateLastCronRunTime($time) {
    $this->state->set('system.cron_last', ($this->time->getCurrentTime() - $time));
  }

  /**
   * Sets the selected option based on the provided time.
   *
   * @param int $time
   *   The time to set the selected option for, typically a Unix timestamp.
   */
  public function setSelectedOption($time) {
    $this->state->set('cron_last_run_manipulate.time', $time);
  }

  /**
   * Retrieves the selected option.
   *
   * This method is responsible for returning the currently selected option
   * from a predefined set of options.
   *
   * @return mixed
   *   The selected option. The type of the returned value depends on the
   *   implementation and the context in which this method is used.
   */
  public function getSelectedOption() {
    return $this->state->get('cron_last_run_manipulate.time') ?? '';
  }

}
